function [Pit,er,it,RR] = glinvev(Eta,C,M,G,Pit,RR)

% Invert marginal parametrization
%
% INPUT:
% Eta:     matrix of marginal parameters for each subject
% C, M, G: matrices costructed by marg_par.m
% Pit:     initial values for the matrix of probabilities
% RR:      corresponding matrix of derivatives
%
% OUTPUT:
% Pit:     final matrix of probabilities
% er:      dummy indicating the presence of errors
% it:      number of iterations for the inversion
% RR:      final matrix of probabilities

% preslimianries
  tol = 10^-10;  
  [ne,n] = size(Eta);
  lastwarn(' ');
% inversion  
  if ne==3,
    Pit = dalev(Eta);
    dis = 0; it = 0;
  else
    np = size(G,1);
    if nargin<5,
      La = zeros(ne,n);
      Pit = ones(np,n)/np;
    else
      La = inv(G'*G)*G'*log(Pit);
    end
    Eta1 = C*log(M*Pit);
    dis = sum(abs(Eta-Eta1))'/ne;
    it = 0; one = ones(np,1);
    while max(dis)>tol && it<10,
      it = it+1;
      Di = Eta-Eta1;
      if it==1 && nargin==6,
        for i = find(dis>tol)',
          La(:,i) = La(:,i)+RR(:,:,i)*Di(:,i);
        end
      else
        DP = 1./(M*Pit);
        for i = 1:n,
          if dis(i)>tol,
            R = (C*((DP(:,i)*Pit(:,i)').*M))*G;
            La(:,i) = La(:,i)+(R\Di(:,i));
          end
        end
        [l1,l2] = lastwarn;
        if length(l2)==21,
          if all(l2=='MATLAB:singularMatrix'), er = 1; return; end
        end
      end
      Pit = G*La; Pit = Pit-ones(np,1)*max(Pit);
      Pit = exp(Pit); Pit = Pit./(one*sum(Pit));
      Eta1 = C*log(M*Pit);
      dis = sum(abs(Eta-Eta1))'/ne;
    end
  end
  if nargout>3,
    I = eye(ne);
    RR = zeros(ne,ne,n); DP = 1./(M*Pit);
    for i = 1:n,
      R = (C*((DP(:,i)*Pit(:,i)').*M))*G;
      RR(:,:,i) = R\I;
    end
    [l1,l2] = lastwarn;
    if length(l2)==21,
      if all(l2=='MATLAB:singularMatrix'), er = 1; return; end
    end
  end
  er = dis>tol;
  if any(any(isnan(Pit))), er = 1; end
  if nargout>3 && any(any(any(any(any(isnan(RR)))))), er = 1; end
%  if n>1,  disp_dis(dis); end
%----------------------------------------------------------------------
function [pv] = dalev(Eta)

%        [pv] = dalev(Eta)
%
% inverts the GLLM parametrization 
% p = valore iniziale
% eta = C*log(A*p)
  Eta = min(max(Eta',-300),300);   %aggiunto
  p1 = exp(Eta(:,1:2)); p1 = p1./(1+p1);
  p11 = prod(p1,2); p110 = p11;
  ind = find(abs(Eta(:,3))<eps);  
  eta3 = Eta(:,3); eta3(ind) = eps;
  psi = exp(eta3);
  a = 1+sum(p1,2).*(psi-1);
  b = -4*psi.*(psi-1).*p11;
  ra = max(a.^2+b,0);  %aggiunto
  p11 = (a-ra.^0.5)./(2*(psi-1));  
  p11(ind) = p110(ind);
  p01 = p1(:,2)-p11;
  p0p = 1-p1(:,1);
  p00 = p0p-p01;
  pv = [p00 p01 p1(:,1)-p11 p11]';