function [C,M,G] = marg_param(lev,type)

%        [C,M,G] = marg_param(lev,type)
%
% Creates matrices C and M for the marginal parametrization
% of the probability vector for a vector of categorical variables
% 
% INPUT:
% lev:  vector containing the number of levels of each variable
% type: vector with elements 'l', 'g', 'c', 'r' indicating the type of logit
%
% OUTPUT:
% C:    matrix of constrats (the first sum(lev)-length(r) elements are
%       referred to univariate logits)
% M:    marginalization matrix with elements 0 and 1
% G:    corresponding design matrix for the corresponding log-linear model

% preliminaries
  r = length(lev);
% create sets of parameters
  S = [fliplr(sq(r,1)); fliplr(sq(r,2));];
% create matrices
  C = []; M = []; G = [];
  for i = 1:size(S,1),
    si = S(i,:); 
    Ci = 1;  % to update matrix C
    for h = 1:r,
      if si(h),
        I = eye(lev(h)-1);
        Ci = kron(Ci,[-I I]);
      end
    end
    C = blkdiag(C,Ci);
    Mi = 1;  % to update matrix M
    for h = 1:r,
      if si(h),
        I = eye(lev(h)-1); T = tril(ones(lev(h)-1)); ze = zeros(lev(h)-1,1);
        switch type(h),
          case 'l',
            Mi = kron(Mi,[I ze; ze I]);
          case 'g',
            Mi = kron(Mi,[T ze; ze T']);
          case 'c',
            Mi = kron(Mi,[I ze; ze T']);
          case 'r',
            Mi = kron(Mi,[T ze; ze I]);
        end          
      else
        Mi = kron(Mi,ones(1,lev(h)));
      end      
    end
    M = [M; Mi];
    Gi = 1;  % for the desgin matrix
    for h = 1:r,
      if si(h),
        Gi = kron(Gi,[-ones(lev(h)-1,1) eye(lev(h)-1)]'/2);
      else
        Gi = kron(Gi,ones(lev(h),1));
      end
    end
    G = [G Gi];
  end
%--------------------------------------------------------------------------  
function [M] = sq(J,t)

% -      [M] = sq(J,<t>)
% - M = matrice formata da tutti i possibili vettori riga composti da 
%       t elementi 1 e J-t elementi 0. Se t  omesso genera tutti i 
%       possibili vettori con vettori di J elementi (1, 0).

  M=[];
  if J == 0, return; end
  if nargin==2,           % se t  presente
    if t == J,          % se sono tutti elementi 1
      M=ones(1,J); 
    elseif t>1,       % se sono sia 1 che 0
      for i=1:J-t+1, 
        S=sq(J-i,t-1);
        r=size(S,1);
        T=[zeros(r,i-1) ones(r,1) S];
        M=[T;M];
      end
    elseif t==1,      % se c' un solo elemento 1       
      M=zeros(J);
      for j=1:J, M(j,J-j+1)=1; end
    else 
      M=zeros(1,J);   % se non ci sono elementi 1
    end
  else                % se t  omesso
    if J==1,
      M = [0;1];
    else
      T = sq(J-1);  nt=size(T,1);
      M = [ zeros(nt,1) T; ones(nt,1) T];
    end
  end
    